﻿/*
Matrix class
21 November 2008
*/

#include <windows.h>
#include <math.h>
#pragma hdrstop

#include "tlib.h"

#include "CVector.h"
#include "CMatrix.h"

using namespace LithDX;



CMatrix::CMatrix()
{
	memset(mData, 0, 4 * 4 * sizeof(double));
	mData[15] = 1;
}



CMatrix CMatrix::operator *(const CMatrix& aRHS) const
{
	CMatrix Result;
	int i = 0;
	for (int y = 0; y < 4; y++)
	{
		for (int x = 0; x < 4; x++)
		{
			double Data = 0;
			for (int z = 0; z < 4; z++)
				Data += mData[z + (y * 4)] * aRHS.mData[x + (z * 4)];
			Result.mData[i++] = Data;
		}
	}
	return Result;
}



CVector CMatrix::Transform(const CVector& aVector) const
{
	CVector Result;
	double w;

	Result.X = mData[0] * aVector.X + mData[4] * aVector.Y + mData[8] * aVector.Z + mData[12];
	Result.Y = mData[1] * aVector.X + mData[5] * aVector.Y + mData[9] * aVector.Z + mData[13];
	Result.Z = mData[2] * aVector.X + mData[6] * aVector.Y + mData[10] * aVector.Z + mData[14];
	w = mData[3] * aVector.X + mData[7] * aVector.Y + mData[11] * aVector.Z + mData[15];
	Result.X /= w;
	Result.Y /= w;
	Result.Z /= w;
	return Result;
}



CMatrix CMatrix::Identity()
{
	CMatrix Result;
	Result(0, 0) = 1;
	Result(1, 1) = 1;
	Result(2, 2) = 1;
	return Result;
}



CMatrix CMatrix::Rotate(double aXAngle, double aYAngle, double aZAngle)
{
	double cx = cos(aXAngle), cy = cos(aYAngle), cz = cos(aZAngle);
	double sx = sin(aXAngle), sy = sin(aYAngle), sz = sin(aZAngle);

	CMatrix Result;
	Result.mData[0] = cy * cz;
	Result.mData[4] = -cy * sz;
	Result.mData[8] = -sy;

	Result.mData[1] = cx * sz - sx * sy * cz;
	Result.mData[5] = cx * cz + sx * sy * sz;
	Result.mData[9] = -sx * cy;

	Result.mData[2] = cx * sy * cz + sx * sz;
	Result.mData[6] = -cx * sy * sz + sx * cz;
	Result.mData[10] = cx * cy;
	return Result;
}



CMatrix CMatrix::Translate(const CVector& aOffset)
{
	CMatrix Result = Identity();
	Result.mData[12] = aOffset.X;
	Result.mData[13] = aOffset.Y;
	Result.mData[14] = aOffset.Z;
	return Result;
}



CMatrix CMatrix::RotateAround(const CVector& aVector, double aAngle)
{
/*
[x^2(1-c)+c    yx(1-c)+zs    xz(1-c)+ys    0]

[yx(1-c)+zs    y^2(1-c)+c    yz(1-c)+xs    0]

[xz(1-c)+ys    yz(1-c)+xs    z^2(1-c)+c    0]

[0             0             0             1]
*/
	double c = 1.0 - cos(aAngle);
	double s = sin(aAngle);

	CMatrix Result;
	Result.mData[0] = aVector.X * aVector.X * c + (1.0 - c);
	Result.mData[1] = aVector.X * aVector.Y * c - aVector.Z * s;
	Result.mData[2] = aVector.X * aVector.Z * c + aVector.Y * s;

	Result.mData[4] = aVector.Y * aVector.X * c + aVector.Z * s;
	Result.mData[5] = aVector.Y * aVector.Y * c + (1.0 - c);
	Result.mData[6] = aVector.Y * aVector.Z * c - aVector.X * s;

	Result.mData[8] = aVector.Z * aVector.X * c - aVector.Y * s;
	Result.mData[9] = aVector.Z * aVector.Y * c + aVector.X * s;
	Result.mData[10] = aVector.Z * aVector.Z * c + (1.0 - c);
	return Result;
}